@@ -1,23 +1,81 @@
 package Type::Tiny;
 
-use 5.006001;
+use 5.008001;
 use strict;
 use warnings;
 
 BEGIN {
-	if ( $] < 5.008 ) { require Devel::TypeTiny::Perl56Compat }
 	if ( $] < 5.010 ) { require Devel::TypeTiny::Perl58Compat }
 }
 
 BEGIN {
 	$Type::Tiny::AUTHORITY  = 'cpan:TOBYINK';
-	$Type::Tiny::VERSION    = '1.016010';
+	$Type::Tiny::VERSION    = '2.008002';
 	$Type::Tiny::XS_VERSION = '0.016';
 }
 
 $Type::Tiny::VERSION    =~ tr/_//d;
 $Type::Tiny::XS_VERSION =~ tr/_//d;
 
+our @InternalPackages = qw(
+	Devel::TypeTiny::Perl56Compat
+	Devel::TypeTiny::Perl58Compat
+	Error::TypeTiny
+	Error::TypeTiny::Assertion
+	Error::TypeTiny::Compilation
+	Error::TypeTiny::WrongNumberOfParameters
+	Eval::TypeTiny
+	Eval::TypeTiny::CodeAccumulator
+	Eval::TypeTiny::Sandbox
+	Exporter::Tiny
+	Reply::Plugin::TypeTiny
+	Test::TypeTiny
+	Type::Coercion
+	Type::Coercion::FromMoose
+	Type::Coercion::Union
+	Type::Library
+	Type::Params
+	Type::Params::Alternatives
+	Type::Params::Parameter
+	Type::Params::Signature
+	Type::Parser
+	Type::Parser::AstBuilder
+	Type::Parser::Token
+	Type::Parser::TokenStream
+	Type::Registry
+	Types::Common
+	Types::Common::Numeric
+	Types::Common::String
+	Types::Standard
+	Types::Standard::_Stringable
+	Types::Standard::ArrayRef
+	Types::Standard::CycleTuple
+	Types::Standard::Dict
+	Types::Standard::HashRef
+	Types::Standard::Map
+	Types::Standard::ScalarRef
+	Types::Standard::StrMatch
+	Types::Standard::Tied
+	Types::Standard::Tuple
+	Types::TypeTiny
+	Type::Tie
+	Type::Tie::ARRAY
+	Type::Tie::BASE
+	Type::Tie::HASH
+	Type::Tie::SCALAR
+	Type::Tiny
+	Type::Tiny::_DeclaredType
+	Type::Tiny::_HalfOp
+	Type::Tiny::Class
+	Type::Tiny::ConsrtainedObject
+	Type::Tiny::Duck
+	Type::Tiny::Enum
+	Type::Tiny::Intersection
+	Type::Tiny::Role
+	Type::Tiny::Union
+	Type::Utils
+);
+
 use Scalar::Util qw( blessed );
 use Types::TypeTiny ();
 
@@ -28,7 +86,7 @@ sub _croak ($;@) { require Error::TypeTiny; goto \&Error::TypeTiny::croak }
 sub _swap { $_[2] ? @_[ 1, 0 ] : @_[ 0, 1 ] }
 
 BEGIN {
-	my $support_smartmatch = 0+ !!( $] >= 5.010001 );
+	my $support_smartmatch = 0+ !!( $] >= 5.010001 && $] <= 5.041002 );
 	eval qq{ sub SUPPORT_SMARTMATCH () { !! $support_smartmatch } };
 	
 	my $fixed_precedence = 0+ !!( $] >= 5.014 );
@@ -55,6 +113,15 @@ BEGIN {
 		$try_xs
 		? sub () { $INC{'Mouse/Util.pm'} and Mouse::Util::MOUSE_XS() }
 		: sub () { !!0 };
+	
+	my $strict_mode = 0;
+	$ENV{$_} && ++$strict_mode for qw(
+		EXTENDED_TESTING
+		AUTHOR_TESTING
+		RELEASE_TESTING
+		PERL_STRICT
+	);
+	*_STRICT_MODE = $strict_mode ? sub () { !!1 } : sub () { !!0 };
 } #/ BEGIN
 
 {
@@ -104,7 +171,7 @@ __PACKAGE__->_install_overloads(
 					return "Type::Tiny::_HalfOp"->new(
 						$op,
 						$param,
-						"Type::Tiny::Union"->new( type_constraints => [ $type, $tc[1] ] ),
+						"Type::Tiny::Union"->new_by_overload( type_constraints => [ $type, $tc[1] ] ),
 					);
 				} #/ if ( blessed $tc[0] eq...)
 			} #/ if ( blessed $tc[0] )
@@ -114,7 +181,7 @@ __PACKAGE__->_install_overloads(
 			}
 		} #/ if ( !_FIXED_PRECEDENCE...)
 		require Type::Tiny::Union;
-		return "Type::Tiny::Union"->new( type_constraints => \@tc );
+		return "Type::Tiny::Union"->new_by_overload( type_constraints => \@tc );
 	},
 	q(&) => sub {
 		my @tc = _swap @_;
@@ -128,7 +195,7 @@ __PACKAGE__->_install_overloads(
 					return "Type::Tiny::_HalfOp"->new(
 						$op,
 						$param,
-						"Type::Tiny::Intersection"->new( type_constraints => [ $type, $tc[1] ] ),
+						"Type::Tiny::Intersection"->new_by_overload( type_constraints => [ $type, $tc[1] ] ),
 					);
 				} #/ if ( blessed $tc[0] eq...)
 			} #/ if ( blessed $tc[0] )
@@ -138,7 +205,7 @@ __PACKAGE__->_install_overloads(
 			}
 		} #/ if ( !_FIXED_PRECEDENCE...)
 		require Type::Tiny::Intersection;
-		"Type::Tiny::Intersection"->new( type_constraints => \@tc );
+		"Type::Tiny::Intersection"->new_by_overload( type_constraints => \@tc );
 	},
 	q(~)  => sub { shift->complementary_type },
 	q(==) => sub { $_[0]->equals( $_[1] ) },
@@ -156,6 +223,7 @@ __PACKAGE__->_install_overloads(
 	q(eq)  => sub { "$_[0]" eq "$_[1]" },
 	q(cmp) => sub { $_[2] ? ( "$_[1]" cmp "$_[0]" ) : ( "$_[0]" cmp "$_[1]" ) },
 	q(0+)  => sub { $_[0]{uniq} },
+	q(/)   => sub { ( _STRICT_MODE xor $_[2] ) ? $_[0] : $_[1] },
 );
 
 __PACKAGE__->_install_overloads(
@@ -226,7 +294,6 @@ our %ALL_TYPES;
 
 my $QFS;
 my $uniq = 1;
-my $subname;
 
 sub new {
 	my $class  = shift;
@@ -236,6 +303,19 @@ sub new {
 		$params{$_} = $params{$_} . '' if defined $params{$_};
 	}
 	
+	my $level = 0;
+	while ( not exists $params{definition_context} and $level < 20 ) {
+		our $_TT_GUTS ||= do {
+			my $g = join '|', map quotemeta, grep !m{^Types::}, @InternalPackages;
+			qr/\A(?:$g)\z/o
+		};
+		my $package = caller $level;
+		if ( $package !~ $_TT_GUTS ) {
+			@{ $params{definition_context} = {} }{ qw/ package file line / } = caller $level;
+		}
+		++$level;
+	}
+	
 	if ( exists $params{parent} ) {
 		$params{parent} =
 			ref( $params{parent} ) =~ /^Type::Tiny\b/
@@ -348,22 +428,29 @@ sub new {
 	# type constraint without needing to load Type::Coercion yet.
 	
 	if ( $params{my_methods} ) {
-		$subname =
-			eval   { require Sub::Util } ? \&Sub::Util::set_subname
-			: eval { require Sub::Name } ? \&Sub::Name::subname
-			: 0
-			if not defined $subname;
-		if ( $subname ) {
-			( Scalar::Util::reftype( $params{my_methods}{$_} ) eq 'CODE' ) && $subname->(
+		require Eval::TypeTiny;
+		Scalar::Util::reftype( $params{my_methods}{$_} ) eq 'CODE'
+			and /\A[^0-9\W]\w+\z/
+			and Eval::TypeTiny::set_subname(
 				sprintf( "%s::my_%s", $self->qualified_name, $_ ),
 				$params{my_methods}{$_},
 			) for keys %{ $params{my_methods} };
-		}
 	} #/ if ( $params{my_methods...})
 	
+	# In general, mutating a type constraint after it's been created
+	# is a bad idea and will probably not work. However some places are
+	# especially harmful and can lead to confusing errors, so allow
+	# subclasses to lock down particular keys.
+	#
+	$self->_lockdown( sub {
+		&Internals::SvREADONLY( $_, !!1 ) for @_;
+	} );
+	
 	return $self;
 } #/ sub new
 
+sub _lockdown {}
+
 sub DESTROY {
 	my $self = shift;
 	delete( $ALL_TYPES{ $self->{uniq} } );
@@ -448,6 +535,7 @@ sub mouse_type           { $_[0]{mouse_type} ||= $_[0]->_build_mouse_type }
 sub deep_explanation     { $_[0]{deep_explanation} }
 sub my_methods           { $_[0]{my_methods} ||= $_[0]->_build_my_methods }
 sub sorter               { $_[0]{sorter} }
+sub exception_class      { $_[0]{exception_class} ||= $_[0]->_build_exception_class }
 
 sub has_parent               { exists $_[0]{parent} }
 sub has_library              { exists $_[0]{library} }
@@ -516,8 +604,8 @@ sub _build_default_message {
 sub _build_name_generator {
 	my $self = shift;
 	return sub {
-		s/[\x00-\x1F]//smg for ( my ( $s, @a ) = @_ );
-		sprintf( '%s[%s]', $s, join q[,], @a );
+		defined && s/[\x00-\x1F]//smg for ( my ( $s, @a ) = @_ );
+		sprintf( '%s[%s]', $s, join q[,], map !defined() ? 'undef' : !ref() && /\W/ ? B::perlstring($_) : $_, @a );
 	};
 }
 
@@ -550,6 +638,21 @@ sub _build_compiled_check {
 	};
 } #/ sub _build_compiled_check
 
+sub _build_exception_class {
+	my $self = shift;
+	return $self->parent->exception_class if $self->has_parent;
+	require Error::TypeTiny::Assertion;
+	return 'Error::TypeTiny::Assertion';
+}
+
+sub definition_context {
+	my $self  = shift;
+	my $found = $self->find_parent(sub {
+		ref $_->{definition_context} and exists $_->{definition_context}{file};
+	});
+	$found ? $found->{definition_context} : {};
+}
+
 sub find_constraining_type {
 	my $self = shift;
 	if ( $self->_is_null_constraint and $self->has_parent ) {
@@ -558,6 +661,21 @@ sub find_constraining_type {
 	$self;
 }
 
+sub type_default {
+	my ( $self, @args ) = @_;
+	if ( exists $self->{type_default} ) {
+		if ( @args ) {
+			my $td = $self->{type_default};
+			return sub { local $_ = \@args; &$td; };
+		}
+		return $self->{type_default};
+	}
+	if ( my $parent = $self->parent ) {
+		return $parent->type_default( @args ) if $self->_is_null_constraint;
+	}
+	return undef;
+}
+
 our @CMP;
 
 sub CMP_SUPERTYPE ()  { -1 }
@@ -893,6 +1011,8 @@ sub inline_assert {
 	my $self = shift;
 	my ( $varname, $typevarname, %extras ) = @_;
 	
+	$extras{exception_class} ||= $self->exception_class;
+	
 	my $inline_check;
 	if ( $self->can_be_inlined ) {
 		$inline_check = sprintf( '(%s)', $self->inline_check( $varname ) );
@@ -938,13 +1058,11 @@ sub inline_assert {
 } #/ sub inline_assert
 
 sub _failed_check {
-	require Error::TypeTiny::Assertion;
-	
 	my ( $self, $name, $value, %attrs ) = @_;
 	$self = $ALL_TYPES{$self} if defined $self && !ref $self;
 	
-	my $exception_class =
-		delete( $attrs{exception_class} ) || "Error::TypeTiny::Assertion";
+	my $exception_class = delete( $attrs{exception_class} )
+		|| ( ref $self ? $self->exception_class : 'Error::TypeTiny::Assertion' );
 	my $callback = delete( $attrs{on_die} );
 
 	if ( $self ) {
@@ -992,7 +1110,7 @@ sub is_parameterized {
 		join ',', map {
 			Types::TypeTiny::is_TypeTiny( $_ )  ? sprintf( '$Type::Tiny::ALL_TYPES{%s}', defined( $_->{uniq} ) ? $_->{uniq} : '____CANNOT_KEY____' ) :
 			ref() eq 'ARRAY'                    ? do { $seen{$_}++ ? '____CANNOT_KEY____' : sprintf( '[%s]', ____make_key( @$_ ) ) } :
-			ref() eq 'HASH'                     ? do { $seen{$_}++ ? '____CANNOT_KEY____' : sprintf( '{%s}', ____make_key( %$_ ) ) } :
+			ref() eq 'HASH'                     ? do { $seen{$_}++ ? '____CANNOT_KEY____' : sprintf( '{%s}', ____make_key( do { my %h = %$_; map +( $_, $h{$_} ), sort keys %h; } ) ) } :
 			ref() eq 'SCALAR' || ref() eq 'REF' ? do { $seen{$_}++ ? '____CANNOT_KEY____' : sprintf( '\\(%s)', ____make_key( $$_ ) ) } :
 			!defined()                          ? 'undef' :
 			!ref()                              ? do { require B; B::perlstring( $_ ) } :
@@ -1038,9 +1156,11 @@ sub is_parameterized {
 				if $compiled;
 			$options{inlined} = $self->inline_generator->( @_ )
 				if $self->has_inline_generator;
+			$options{type_default} = $self->{type_default_generator}->( @_ )
+				if exists $self->{type_default_generator}; # undocumented
 			exists $options{$_} && !defined $options{$_} && delete $options{$_}
 				for keys %options;
-				
+			
 			$P = $self->create_child_type( %options );
 			
 			if ( $self->has_coercion_generator ) {
@@ -1065,6 +1185,23 @@ sub is_parameterized {
 	} #/ sub parameterize
 }
 
+sub check_parameter_count_for_parameterized_type {
+	my ( $library, $type_name, $args, $max_args, $min_args ) = @_;
+	$args = @$args if ref $args;
+	
+	if ( ( defined $max_args and $args > $max_args ) or ( defined $min_args and $args < $min_args ) ) {
+		require Error::TypeTiny::WrongNumberOfParameters;
+		Error::TypeTiny::WrongNumberOfParameters->throw(
+			target => "$library\::$type_name\[]",
+			( defined $min_args ) ? ( minimum => $min_args ) : (),
+			( defined $max_args ) ? ( maximum => $max_args ) : (),
+			got => $args,
+		);
+	}
+	
+	return;
+}
+
 sub child_type_class {
 	__PACKAGE__;
 }
@@ -1162,6 +1299,59 @@ sub _build_mouse_type {
 	return $r;
 } #/ sub _build_mouse_type
 
+sub exportables {
+	my ( $self, $base_name, $tag ) = ( shift, @_ ); # $tag is undocumented
+	if ( not $self->is_anon ) {
+		$base_name ||= $self->name;
+	}
+	$tag ||= 0;
+
+	my @exportables;
+	return \@exportables if ! $base_name;
+
+	require Eval::TypeTiny;
+
+	push @exportables, {
+		name => $base_name,
+		code => Eval::TypeTiny::type_to_coderef( $self ),
+		tags => [ 'types' ],
+	} if $tag eq 'types' || !$tag;
+
+	push @exportables, {
+		name => sprintf( 'is_%s', $base_name ),
+		code => $self->compiled_check,
+		tags => [ 'is' ],
+	} if $tag eq 'is' || !$tag;
+
+	push @exportables, {
+		name => sprintf( 'assert_%s', $base_name ),
+		code => $self->_overload_coderef,
+		tags => [ 'assert' ],
+	} if $tag eq 'assert' || !$tag;
+
+	push @exportables, {
+		name => sprintf( 'to_%s', $base_name ),
+		code => $self->has_coercion && $self->coercion->frozen
+			? $self->coercion->compiled_coercion
+			: sub ($) { $self->coerce( $_[0] ) },
+		tags => [ 'to' ],
+	} if $tag eq 'to' || !$tag;
+
+	return \@exportables;
+}
+
+sub exportables_by_tag {
+	my ( $self, $tag, $base_name ) = ( shift, @_ );
+	my @matched = grep {
+		my $e = $_;
+		grep $_ eq $tag, @{ $e->{tags} || [] };
+	} @{ $self->exportables( $base_name, $tag ) };
+	return @matched if wantarray;
+	_croak( 'Expected to find one exportable tagged "%s", found %d', $tag, scalar @matched )
+		unless @matched == 1;
+	return $matched[0];
+}
+
 sub _process_coercion_list {
 	my $self = shift;
 	
@@ -1256,7 +1446,7 @@ sub coercibles {
 sub isa {
 	my $self = shift;
 	
-	if ( $INC{"Moose.pm"}
+	if ( $INC{"Moose/Meta/TypeConstraint.pm"}
 		and ref( $self )
 		and $_[0] =~ /^(?:Class::MOP|MooseX?::Meta)::(.+)$/ )
 	{
@@ -1269,7 +1459,7 @@ sub isa {
 		
 		my $inflate = $self->moose_type;
 		return $inflate->isa( @_ );
-	} #/ if ( $INC{"Moose.pm"} ...)
+	} #/ if ( $INC{"Moose/Meta/TypeConstraint.pm"} ...)
 	
 	if ( $INC{"Mouse.pm"}
 		and ref( $self )
@@ -1317,7 +1507,7 @@ sub can {
 	return $can if $can;
 	
 	if ( ref( $self ) ) {
-		if ( $INC{"Moose.pm"} ) {
+		if ( $INC{"Moose/Meta/TypeConstraint.pm"} ) {
 			my $method = $self->moose_type->can( @_ );
 			return sub { shift->moose_type->$method( @_ ) }
 				if $method;
@@ -1348,7 +1538,7 @@ sub AUTOLOAD {
 	return if $m eq 'DESTROY';
 	
 	if ( ref( $self ) ) {
-		if ( $INC{"Moose.pm"} ) {
+		if ( $INC{"Moose/Meta/TypeConstraint.pm"} ) {
 			my $method = $self->moose_type->can( $m );
 			return $self->moose_type->$method( @_ ) if $method;
 		}
@@ -1559,60 +1749,63 @@ Type::Tiny - tiny, yet Moo(se)-compatible type constraint
 
 =head1 SYNOPSIS
 
- use v5.12;
- use strict;
- use warnings;
- 
- package Horse {
-   use Moo;
-   use Types::Standard qw( Str Int Enum ArrayRef Object );
-   use Type::Params qw( compile );
-   use namespace::autoclean;
-   
-   has name => (
-     is       => 'ro',
-     isa      => Str,
-     required => 1,
-   );
-   has gender => (
-     is       => 'ro',
-     isa      => Enum[qw( f m )],
-   );
-   has age => (
-     is       => 'rw',
-     isa      => Int->where( '$_ >= 0' ),
-   );
-   has children => (
-     is       => 'ro',
-     isa      => ArrayRef[Object],
-     default  => sub { return [] },
-   );
-   
-   sub add_child {
-     state $check = compile( Object, Object );  # method signature
-     
-     my ($self, $child) = $check->(@_);         # unpack @_
-     push @{ $self->children }, $child;
-     
-     return $self;
-   }
- }
- 
- package main;
- 
- my $boldruler = Horse->new(
-   name    => "Bold Ruler",
-   gender  => 'm',
-   age     => 16,
- );
- 
- my $secretariat = Horse->new(
-   name    => "Secretariat",
-   gender  => 'm',
-   age     => 0,
- );
- 
- $boldruler->add_child( $secretariat );
+  use v5.36;
+  
+  package Horse {
+    use Moo;
+    use Types::Standard qw( Str Int Enum ArrayRef Object );
+    use Type::Params qw( signature_for );
+    use namespace::autoclean;
+    
+    has name => (
+      is       => 'ro',
+      isa      => Str,
+      required => 1,
+    );
+    
+    has gender => (
+      is       => 'ro',
+      isa      => Enum[qw( f m )],
+    );
+    
+    has age => (
+      is       => 'rw',
+      isa      => Int->where( '$_ >= 0' ),
+    );
+    
+    has children => (
+      is       => 'ro',
+      isa      => ArrayRef[Object],
+      default  => sub { return [] },
+    );
+    
+    # method signature
+    signature_for add_child => (
+      method     => Object,
+      positional => [ Object ],
+    );
+    
+    sub add_child ( $self, $child ) {
+      push $self->children->@*, $child;
+      return $self;
+    }
+  }
+  
+  package main;
+  
+  my $boldruler = Horse->new(
+    name    => "Bold Ruler",
+    gender  => 'm',
+    age     => 16,
+  );
+  
+  my $secretariat = Horse->new(
+    name    => "Secretariat",
+    gender  => 'm',
+    age     => 0,
+  );
+  
+  $boldruler->add_child( $secretariat );
 
 =head1 STATUS
 
@@ -1651,6 +1844,11 @@ objects which are compatible with Moo, Moose and Mouse.
       has favourite_number => (is => "ro", isa => $NUM);
    }
 
+Type::Tiny conforms to L<Type::API::Constraint>,
+L<Type::API::Constraint::Coercible>,
+L<Type::API::Constraint::Constructor>, and
+L<Type::API::Constraint::Inlinable>.
+
 Maybe now we won't need to have separate MooseX, MouseX and MooX versions
 of everything? We can but hope...
 
@@ -1757,6 +1955,12 @@ You may pass C<< coercion => 1 >> to the constructor to inherit coercions
 from the constraint's parent. (This requires the parent constraint to have
 a coercion.)
 
+If an arrayref is passed to the constructor (C<< coercion => [ ... ] >>),
+then the coercion object will be lazily built and this array will be fed to
+its C<add_type_coercions> method. If a coderef is passed to the constructor
+(C<< coercion => sub { ... } >>), then the coercion object will be lazily built
+and this code will be used as a coercion from B<Any>.
+
 =item C<< sorter >>
 
 A coderef which can be passed two values conforming to this type constraint
@@ -1769,11 +1973,60 @@ The idea is to allow for:
   @sorted = Int->sort( 2, 1, 11 );    # => 1, 2, 11
   @sorted = Str->sort( 2, 1, 11 );    # => 1, 11, 2 
 
+=item C<< type_default >>
+
+A coderef which returns a sensible default value for this type. For example,
+for a B<Counter> type, a sensible default might be "0":
+
+  my $Size = Type::Tiny->new(
+    name          => 'Size',
+    parent        => Types::Standard::Enum[ qw( XS S M L XL ) ],
+    type_default  => sub { return 'M'; },
+  );
+  
+  package Tshirt {
+    use Moo;
+    has size => (
+      is       => 'ro',
+      isa      => $Size,
+      default  => $Size->type_default,
+    );
+  }
+
+Child types will inherit a type default from their parent unless the child
+has a C<constraint>. If a type neither has nor inherits a type default, then
+calling C<type_default> will return undef.
+
+As a special case, this:
+
+  $type->type_default( @args )
+
+Will return:
+
+  sub {
+    local $_ = \@args;
+    $type->type_default->( @_ );
+  }
+
+Many of the types defined in L<Types::Standard> and other bundled type
+libraries have type defaults, but discovering them is left as an exercise
+for the reader.
+
 =item C<< my_methods >>
 
 Experimental hashref of additional methods that can be called on the type
 constraint object.
 
+=item C<< exception_class >>
+
+The class used to throw an exception when a value fails its type check.
+Defaults to "Error::TypeTiny::Assertion", which is usually good. This class
+is expected to provide a C<throw_cb> method compatible with the method of
+that name in L<Error::TypeTiny>.
+
+If a parent type constraint has a custom C<exception_class>, then this
+will be "inherited" by its children.
+
 =back
 
 =head3 Attributes related to parameterizable and parameterized types
@@ -1833,8 +2086,9 @@ Optional.
 =item C<< coercion_generator >>
 
 A coderef which generates a new L<Type::Coercion> object based on parameters.
-Called with the same parameters and package variables as the
-C<constraint_generator>. Expected to return a blessed object.
+It is passed the parent type, child type, and list of parameters. It should
+have access to the same package variables as the C<constraint_generator>.
+Expected to return a blessed object.
 
 Optional.
 
@@ -1864,6 +2118,16 @@ Coderef to validate a value (C<< $_[0] >>) against the type constraint.
 This coderef is expected to also handle all validation for the parent
 type constraints.
 
+=item C<< definition_context >>
+
+Hashref of information indicating where the type constraint was originally
+defined. Type::Tiny will generate this based on C<caller> if you do not
+supply it. The hashref will ordinarily contain keys C<"package">, C<"file">,
+and C<"line">.
+
+For parameterized types and compound types (e.g. unions and intersections),
+this may not be especially meaningful information.
+
 =item C<< complementary_type >>
 
 A complementary type for this type. For example, the complementary type
@@ -2291,15 +2555,38 @@ See also L<Type::API::Constraint>, etc.
 
 =item C<< TIESCALAR >>, C<< TIEARRAY >>, C<< TIEHASH >>
 
-These are provided as hooks that wrap L<Type::Tie>. (Type::Tie is distributed
-separately, and can be used with non-Type::Tiny type constraints too.) They
-allow the following to work:
+These are provided as hooks that wrap L<Type::Tie>. They allow the following
+to work:
 
    use Types::Standard qw(Int);
    tie my @list, Int;
    push @list, 123, 456;   # ok
    push @list, "Hello";    # dies
 
+=item C<< exportables( $base_name ) >>
+
+Returns a list of the functions a type library should export if it contains
+this type constraint.
+
+Example:
+
+  [
+    { name => 'Int',        tags => [ 'types' ],  code => sub { ... } },
+    { name => 'is_Int',     tags => [ 'is' ],     code => sub { ... } },
+    { name => 'assert_Int', tags => [ 'assert' ], code => sub { ... } },
+    { name => 'to_Int',     tags => [ 'to' ],     code => sub { ... } },
+  ]
+
+C<< $base_name >> is optional, but allows you to get a list of exportables
+using a specific name. This is useful if the type constraint has a name
+which wouldn't be a legal Perl function name.
+
+=item C<< exportables_by_tag( $tag, $base_name ) >>
+
+Filters C<exportables> by a specific tag name. In list context, returns all
+matching exportables. In scalar context returns a single matching exportable
+and dies if multiple exportables match, or none do!
+
 =back
 
 The following methods exist for Moose/Mouse compatibility, but do not do
@@ -2319,6 +2606,21 @@ anything useful.
 
 =back
 
+=head2 Functions
+
+=over
+
+=item *
+
+C<< check_parameter_count_for_parameterized_type( $lib, $typename, $args, $max, $min ) >>
+
+Utility function used by some types from Types::Standard, etc.
+
+Will throw a L<Error::TypeTiny::WrongNumberOfParameters> exception referencing
+C<< "$lib::\$typename\[]" >> if C<< $args >> is greater than C<< $max >> or
+less than C<< $min >>, if they're defined. If C<< $args >> is an arrayref,
+will use the length of the array.
+
 =head2 Overloading
 
 =over
@@ -2362,6 +2664,12 @@ See L<Type::Tiny::Union>.
 The C<< & >> operator is overloaded to build the intersection of two type
 constraints. See L<Type::Tiny::Intersection>.
 
+=item *
+
+The C<< / >> operator provides magical L<Devel::StrictMode> support.
+If C<< $ENV{PERL_STRICT} >> (or a few other environment variables) is true,
+then it returns the left operand. Normally it returns the right operand.
+
 =back
 
 Previous versions of Type::Tiny would overload the C<< + >> operator to
@@ -2478,7 +2786,7 @@ Thanks to Matt S Trout for advice on L<Moo> integration.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013-2014, 2017-2022 by Toby Inkster.
+This software is copyright (c) 2013-2014, 2017-2025 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
